# read version from installed package
from importlib.metadata import version
__version__ = version("beautifulplots")


from .beautifulplots import plot_defaults
from .beautifulplots import set_axisparams
from .beautifulplots import get_kwargs
from .beautifulplots import set_yaxis_format
from .barplot import barplot
from .lineplot import lineplot
from .scatterplot import scatterplot



import matplotlib.pyplot as plt
import seaborn as sns
from matplotlib.colors import Normalize
import matplotlib as mpl

import  beautifulplots.beautifulplots as bp  


def barplot(df, bar_columns, bar_values, barcurrency=None, barorientation="v", bardataformat="1.2f",
            y2=None,  estimator=sum, estimator2=sum,
            ax=None, bardatalabels=False, test_mode=False, bardatafontsize=14,
            **kwargs):
    """Bar plot function designed for ease of use and aesthetics. 
    The underlying barplot is ased on the Seaborn with additions, such as secondary axis, data labels,
    and improved default parameters. Refer to beautifulplots plot_defaults for a complete list of options.
    
    Args:
        df (DataFrame): The input DataFrame containing colums corresponding to bar_plot values ("bar_values") and column names (see examples in documentation)
            
        bar_columns: Datafrae columns corresponding to bar column names
            
        bar_values: Dataframe column corresponding to bar column values
            
        ax (axis): matplotlib axis (optional), default = None. If axis is None, then create a matplolib figure, axis to host the barplot
            
        color: Matplotlib compatabile color name as text or RGB values, for example,  color = [51/235,125/235,183/235].
            
        palette: Matplotlib compatible color palette name, for example, "tab20"
            
        hue: Name of hue dimension variable (i.e., DataFrame column name)
            
        ci: Seaborn confidence interval parameter: float, sd, or None, default = None
            
        barorientation: default = v (vertical), or h (horizontal)
            
        barcurrency: default = False (bar values do not represent currency). True (bar values represent currency, append $ to the value)
            
        bardatalabels (Boolean): default = False (data labels not included)
        
        estimater: default = sum. Specifies how to aggregate plot bar data.
        
        estimator2: default = None. Summarize y2 asis daa. Default is no aggregation, do not summarize y2 axis data.
        
        additional options:  see beautifulplot.plot_defaults for additional input variables.
        

    Returns:
        returns True if processing completes succesfully (without errors).
    """
        
    plot_options = bp.get_kwargs(**kwargs)
    
    errorbar = plot_options['errorbar']
    errorbar2 = plot_options['errorbar2']
    alpha = plot_options['alpha']
    alpha2 = plot_options['alpha2']
    hue = plot_options['hue']
    palette = plot_options['palette']
    palette2 = plot_options['palette2']
    marker2 = plot_options['marker2']
    marker2 = plot_options['marker2']
    markers2 = plot_options['markers']
    style2 = plot_options['style2']
    color = plot_options['color']
    color2 = plot_options['color2']
    y2label=plot_options['y2_axis_label']
    y2axisformat = plot_options['y2_axis_format']
    y2currency = plot_options['y2_currency']


    # if no hue then only one color
    # if hue == None and color==None : color = [51/235,125/235,183/235] if plot_options['color'] == None else plot_options['color']

    if barorientation == 'v': x,y = bar_columns, bar_values
    else: x,y = bar_values, bar_columns
    

    if ax == None: 
        mpl.rcParams.update(mpl.rcParamsDefault) # reset plot/figure parameters
        plt.style.use(plot_options['pltstyle'])
        fig,_ax = plt.subplots(nrows=1, ncols=1, figsize=plot_options['figsize']) 
    else: _ax = ax
        
        
    g=sns.barplot(x=x, y=y, hue=hue, color = color, palette=palette, data=df, ax = _ax,
                  orient=barorientation, errorbar=errorbar, estimator=estimator, alpha=alpha)
    
    # Bar labels ... iterate with hue
    # Matplotlib
      # https://matplotlib.org/stable/gallery/lines_bars_and_markers/bar_label_demo.html#sphx-glr-gallery-lines-bars-and-markers-bar-label-demo-py
      # Geeks for Geeks bar data labels
      # https://www.geeksforgeeks.org/how-to-show-values-on-seaborn-barplot/
    
    if  bardatalabels == True:
        f = bardataformat
        for i in g.containers: 
            if barcurrency!=None:
                g.bar_label(i,fontsize=bardatafontsize,labels=[f'{barcurrency}{x:{f}}' for x in i.datavalues] )
            else:
                g.bar_label(i,fontsize=bardatafontsize,labels=[f'{x:{f}}' for x in i.datavalues] )
   
    # yaxis tick label format
    # https://matplotlib.org/stable/gallery/pyplots/dollar_ticks.html
    # x or y format same as bars ... since this could be v or h graph
        #y_ticks = _ax.get_yticks()
        
    f='{x:'+ bardataformat  +'}'
    if isinstance(barcurrency,str): f= barcurrency + f
    if barorientation=='v':
        _ax.yaxis.set_major_formatter(f)
    if barorientation=='h':
        _ax.xaxis.set_major_formatter(f)
 
   # secondary y-axis
    if y2 != None:
       
        # make sure y2 and marker2 are iterable
        if not isinstance(y2,list): y2 = [y2]
        if marker2 == None:
            if not isinstance(marker2,list): marker2 = len(y2)*[marker2]
        else:
            if not isinstance(marker2,list): marker2 =[marker2]

        _ax2 = _ax.twinx()
        
        
        for _y2,_marker2 in zip(y2,marker2):
            label = _y2 if hue == None else None
            if plot_options['palette2'] !=None:
                g = sns.lineplot(data=df,x=x, y =_y2, hue=hue, palette=palette2,  ax=_ax2, label=label,
                                 alpha = alpha2,errorbar = errorbar2, marker=_marker2, estimator=estimator2,
                                 markers=markers2, style=style2)
            elif plot_options['color2'] !=None:
                g = sns.lineplot(data=df,x=x, y=_y2, hue=hue, color=color2,  ax=_ax2, label=label,
                                 alpha=alpha2, errorbar=errorbar2, marker=_marker2, estimator=estimator2,
                                 markers=markers2, style=style2)
            else:
                g = sns.lineplot(data=df,x=x, y=_y2, hue=hue, ax=_ax2, label=label,
                                 alpha=alpha2, errorbar=errorbar2, marker=_marker2, estimator=estimator2,
                                 markers=markers2, style=style2) 
                
        _ax2.grid(visible=None)  
    
    # set axis params
    bp.set_axisparams(plot_options,_ax,g)  # axis parameters from the plot_options dictionary
    
    # y2 axis params
    if y2 != None:
        plot_options["y_label"]=y2label
        bp.set_axisparams(plot_options,_ax2,g)  # axis parameters
    
        # set ylims 2 after general axis parameters 
        if plot_options['y_lims2'] != None:
            _ax2.set_ylim(plot_options['y_lims2'])
        
        # axis 2 legend
        handles, labels = _ax2.get_legend_handles_labels()
        if y2 != None and handles==True:
            _ax2.legend( loc=plot_options['legend_loc2'], prop={'size': plot_options['legend_size']})
            
        bp.set_yaxis_format(_ax2,y2axisformat, y2currency)
    

    if ax==None and test_mode==False: plt.show() # if barplot created the figure then plt.show()
    
    return None



import matplotlib.pyplot as plt
import seaborn as sns
from matplotlib.colors import Normalize
from matplotlib.cbook import boxplot_stats
import matplotlib as mpl 


def plot_defaults():
    """Dictionary of plot parameters. Each parameter coresponds and corresponding value. 
    See also get_plot_options for extracting plot options from **kwargs. 

    **Axis - x, y, and plot area parameters**
    
    Args:
        df (DataFrame): The input DataFrame containing colums corresponding to bar values and columns.
 
        title (String): corresponds to the axis title. default = ''
            
        title_fontsize: font size of the axis title, default = 18
         
        x_lims: (xmin, xmax), minimum and maximum x-values of the axis. default = None, in which case the min and max are set automatically by matplotlib.
        
        y_lims: (ymin,ymax), minimum and maximum y-values of the axis. default = None, in which case the min and max are set automatically by matplotlib.
        
        y2_lims: (ymin,ymax), minimum and maximum y-values of the secondary axis. default = None, in which case the min and max are set automatically by matplotlib.
        
        x_label_fontsize: default = 16
        
        x_tick_labelsize: default = 16
        
        x_tick_fontsize: default = 16
        
        x_tick_rotation: default = 0
        
        y_label_fontsize: ylabel font size, default = 16
            
        y_tick_fontsize: xtick label font size, default = 16
        
        y_tick_rotation:  Rotation of the xtick label, default = 0
        

        
    **Plot attributes - Line, Bar, Scatter Plots**
    
    Args:
        
        marker: Matplotlib line markers. default = None (Matplotlib default).
            
        marker2: Secondary axis, Matplotlib line marker. default = None (Matplotlib default).
        
        marker: sns line markers. default = None (sns default).
            
        marker2: Secondary axis, sns line markers. default = None (sns default).
            
        y_axis_currency (Boolean): Boolean default = False.
            
        y_tick_format (String): String default = None (Matplotlib default).
            
        alpha (fraction): Trasnparancy (opacity), default = None (not transparent)
            
        alpha2 (fraction): Secondary axis, default = 0.5, 50% opacity
        
        legend_labels (list): Overide default legend labels. default = None (do not override)
        
        legend_loc(String): Matplotlib legend location, for example, upper right , default = "best".
        
        legend_loc2 (String): Secondary axis legend location, for example, upper right , default = best.
        
        legend_fontsize: legend font size, default = 16
        
        estimator: seaborn barplot summary estimator, default = sum
            
        estimator2: secondary axis, seaborn barplot summary estimator, default = sum
            
        color: default = None, indicateing Matplolib default (Matplotlib default)
            
        color2: secondary axis line, bar or color. defualt = None (Matplotlib default)
            
        palette: colormap, default = None
            
        palette2: colormap, default = None, secondary y axis.
            
        hue: dimension value for corresponding Seaborn graphs, default = None.
        
        errorbar: Seaborn errorbar parameter: float, sd, or None
            
        errorbar2: Seaborn errorbar parameter second y axis: float, sd, or None
            
        x_label (String): xlabel title, default = ''
            
        y_label (String):  y lable title, default = ''
        
        h_line (int):  y-value, corresponding to horizontal line, default = None
                
        h_line_label (String):  y value, corresponding to horizontal line, default = None
        
        v_line (int):  x-value, corresponding to vertical line, default = None (no line)
        
        v_line_label (String):  x value, corresponding to vertical line, default = None (no line)

        y_scale (String): scale for the y-axis (e.g., yscale = "log"), default = None (default is linear)
        
        x_scale (String): scale for the x-axis (e.g., xscale = "log"), default = None (default is linear)
     
    **Plots and subplots**
    
    Parameters corresponding to the plot or subplot characteristics. They are used when matplot_helpers
    functions create the plot figure and axis, otherewise, these parameters do not affect the plot.
    
    Args:
    
        plot_style (String): matplotlib plot style
            
        fig_size: total size (height, width) in inches of the figure, including total plotting area of all subplots and spacing
        
        w_space: width space (horizontal) between subplots, default wspace = 0.2
        
        h_space: height space (vertical) between subplots, default hspace = 0.2
        
    **Annotations**
    
    Args:
        annotate (List of Strings and positions): default = None.  List of Strings with corresponding (x,y) positions.
        
        annotate_fontsize (int): default = 16. Fontsize for each annotation.
            
    **Returns**
    
        Dictionary: {parameter1:value1, parameter2:value2, ... }. 
        Pairs of plat parameters and corresponding values.
    """    

    plot_defaults = {
        
    # axis, plot and subplot
    'pltstyle': 'ggplot',
    'figsize' : None,
    'w_space': 0.2,
    'h_space': 0.2,
    'share_x':False,
    'title' :None,               #  list ot titles, 1 per axis  #  list of x,y tuples
    'y_lims' :None,               # list of two-tuples ylims (lower, upper)
    'x_lims' :None,
    'y_lims2':None,
    "yfb": None,
    "yfb_color": "red",
    "yfb_alpha": 0.5,
    'y_currency':None,
    'y2_currency':None,
    'y_axis_format':"1.2f",
    'y2_axis_format':"1.2f",
    'x_label_fontsize':16,
    'x_tick_labelsize':16,
    'x_tick_fontsize': 16,
    'x_tick_rotation':0,
    'y_label_fontsize':16,
    'y_tick_fontsize':16,
    'y_tick_rotation':0,
    'title_fontsize':18,
    'legend':True,
    'legend_labels':None,
    'legend_loc':'best',
    'legend_loc2':'best',
    'legend_fontsize':16,
    'x_axis_label': None,
    'y_axis_label': None,
    'y2_axis_label': None,
    'marker':None,
    'marker2':None,
    'markers':None,
    'markers2':None,
    "linestyle":"-",
    "linestyle2":"-",
    'style':None,
    'style2':None,
    'color':None,   # color designation for the corresponding graph
    'color2':None,   # color secondary y axis
    'palette':None,  
    'palette2':None,  # palette secondary y axis
    'hue':None,   # dimensional value for corresponding Seaborn graphs
    'errorbar':None, # confidence parameter for pimary axis
    'errorbar2': None, # confidence parameter for secondary axis


    # Lines, bar, and Scatter plots
    'ytick_format': None,
    'alpha': None,
    'alpha2': 0.5,
    'estimator':"sum",
    'estimator2':"sum",
    'h_line':None,
    "v_line":None,
    "h_line_label": None,
    "v_line_label": None,
    "y_scale":None,
    "x_scale":None,
    
    
    # annotations
    "annotate":None,
    "annotate_fontsize":16

    }

    return plot_defaults

def set_axisparams(options_dict,ax,g): 
    """Receives as input a dictionary of plot options and applies the options to the maxtplotlib axis and graph.

    Args:
        options_dict (dictionary): dictionar containint plot options. Each key, value pair corresponds to a plot parameter
        ax (axis): matplotlib axis to apply the plot options
        g (graph): matplotlib graph to apply the plot options

    Returns:
        None: returns None if the function completes without errors.
    """ 
    from beautifulplots import beautifulplots as bp 
    
    annotate=options_dict['annotate']
    annotate_fontsize=options_dict['annotate_fontsize']
    title=options_dict['title']
    title_fontsize=options_dict['title_fontsize']
    legend=options_dict['legend']
    legendloc=options_dict['legend_loc']
    legendfontsize=options_dict['legend_fontsize']
    xlabel=options_dict['x_axis_label']
    xlabelfontsize=options_dict['x_label_fontsize']
    xlims=options_dict['x_lims']
    xtickfontsize=options_dict['x_tick_fontsize']
    xtickrotation=options_dict['x_tick_rotation']
    ylabel=options_dict['y_axis_label']
    ylabelfontsize=options_dict['y_label_fontsize']
    ytickfontsize=options_dict['y_tick_fontsize']
    ytickrotation=options_dict['y_tick_rotation']
    ylims=options_dict['y_lims']
    hline=options_dict['h_line']
    vline=options_dict['v_line']
    hline_label=options_dict['h_line_label']
    vline_label=options_dict['v_line_label']
    yscale=options_dict['y_scale']
    xscale=options_dict['x_scale']
    
    ax.set_xlabel(xlabel,fontsize=xlabelfontsize)
    ax.set_ylabel(ylabel,fontsize=ylabelfontsize)

    for x_tick in ax.get_xticklabels():
        x_tick.set_fontsize(xtickfontsize)
        x_tick.set_rotation(xtickrotation)

    for y_tick in ax.get_yticklabels():
        y_tick.set_fontsize(ytickfontsize)
        y_tick.set_rotation(ytickrotation)


    ax.set_title(title, fontsize=title_fontsize)

    if ylims != None:
        ax.set_ylim(ylims[0],ylims[1])

    if xlims != None:
        ax.set_xlim(xlims[0],xlims[1])

    # if legend then set fontsize ... otherwise get warning

    
    handles, labels = ax.get_legend_handles_labels()
    if handles: 
        ax.legend().set_visible(legend)
    if handles and legend == True: 
        if legend == True: ax.legend( loc=legendloc, prop={'size': legendfontsize})     
        
    # horizontal line(s)
    if hline != None:
        if type(hline) is not list and type(hline) is not tuple: hline = [hline]
        for hl in hline:
            ax.axhline(y=hl, xmin=0.0, label=hline_label, xmax=1.0, color='k',lw=1)
        
    if vline != None:
        if type(vline) is not list and type(vline) is not tuple: vline = [vline]
        for vl in vline:
            ax.axvline(x=vl, ymin=0.0, ymax=1.0, label=vline_label, color='k',lw=1)
        
    if yscale == "log":
        g.set(yscale="log")
        
    if xscale == "log":
        g.set(xscale="log")
        
    if annotate != None:
        for a in annotate: 
            ax.annotate(a[0],xy=a[1], fontsize=annotate_fontsize)
    
    # vertical line(s)

    return None

def get_kwargs(**kwargs):   
    """process **kwargs options corresponding to the plot_defaults dictionary (see above)
    If a beautifulplots plot_option dictionary key is 
    contained in the **kwargs then the plot_defautls[key] value is replaced 
    with that found in **kwargs. 

    Returns:
        Dictionary: {parameter1:value1, parameter2:value2, ...} dictionary corresponding to plot options 
    """   
       
    plot_options = plot_defaults() # returns a dictionary of defaut matplotlib parameters

    # interate through the parameters in the plot_options dictionary
    # find the key in the kwargs, otherwise the paremeer = default
    # if the kwargs parameter != default then update plot_options
    for key in plot_options:
        default = plot_options[key]
        kwarg_value = kwargs.get(key, default) # parameter = kwarg  or default
        if kwarg_value != default: plot_options[key] = kwarg_value # update plot_option 
        
    return plot_options


def set_yaxis_format(ax,yaxisformat="1.2f", ycurrency=None, labelcolor='black', which='major'):
    
    # https://matplotlib.org/stable/gallery/pyplots/dollar_ticks.html
    # Use automatic StrMethodFormatter

    
    f='{x:'+ yaxisformat  +'}'
    if ycurrency !=None: f = ycurrency + f
    ax.yaxis.set_major_formatter(f)
    ax.yaxis.set_tick_params(which=which, labelcolor=labelcolor)
    
    return None


import matplotlib.pyplot as plt
import seaborn as sns
import matplotlib as mpl

import  beautifulplots.beautifulplots as bp  

def lineplot(df, x, y, y2=None,  ax=None, test_mode=False, estimator=None, estimator2=None, 
             **kwargs):
    """Lineplot function designed for ease of use and aesthetics. Based on the
    Seaborn lineplot, with additions such as secondary axis, ease of use, and 
    improved default parameters. Refer to beautiful plot_defaults for full list of options.
 
    Args:
        df (Dataframe): The input DataFrame containing colums corresponding to x and y     
            
        x: Dataframe column corresponding to the lineplot x-axis
            aldfsd;lfj
        
        y: Dolumn or list of columns corresponding to the lineplot y-axis
            
        y2: Column or list of columns correspondng to the secondary axis, default = None 
        
        yaxisformat: default = "1.2f" 
        
        ycurrency: default = None. Primary y-axis. For example = "$" to prepend dollar symbol.
        
        y2currency: default = None. Secondary y-axis. For example = "$" to prepend dollar symbol.
        
        marker: default = None. Primary y-axis. Matplotlib line marker. If y is a list, then marker must be a list of markers.
        
        marker2: default = None. Secondary y-axis. Matplotlib line marker. If y2 is a list, then marker2 must be a list of markers.
        
        estimator: Specifies how to summarize data corresponding to y-axis. Defaults to plot all data points do not summarize.
        
        estimator2: Specifies how to summarize data corresponding to y2-axis. Defaults to plot all data points do not summarize.
        
        additional options:  see beautifulplot.plot_defaults for additional input variables.

    Returns:
        returns None if processing completes succesfully (without errors).
    """ 
    #***
 
    plot_options = bp.get_kwargs(**kwargs)
    
    
    # get plot_options 
    hue = plot_options['hue']
    errorbar = plot_options['errorbar']
    errorbar2 = plot_options['errorbar2']
    alpha = plot_options['alpha']
    alpha2 = plot_options['alpha2']
    hue = plot_options['hue']
    palette = plot_options['palette']
    palette2 = plot_options['palette2']
    color = plot_options['color']
    color2 = plot_options['color2']
    marker = plot_options['marker']
    marker2 = plot_options['marker2']
    markers = plot_options['marker']
    markers2 = plot_options['marker2']
    linestyle = plot_options["linestyle"]
    linestyle2 = plot_options["linestyle2"]
    style = plot_options['style']
    style2 = plot_options['style2']
    ycurrency = plot_options['y_currency']
    y2currency = plot_options['y2_currency']
    ylabel = plot_options['y_axis_label']
    y2label = plot_options['y2_axis_label']
    yaxisformat = plot_options['y_axis_format']
    y2axisformat = plot_options['y2_axis_format']
    yfb = plot_options['yfb']
    yfb_color = plot_options["yfb_color"]
    yfb_alpha = plot_options["yfb_alpha"]
    
    
    # get back to the default plot options
    if ax == None: 
        mpl.rcParams.update(mpl.rcParamsDefault) # reset plot/figure parameters
        plt.style.use(plot_options['pltstyle'])
        fig,_ax = plt.subplots(nrows=1, ncols=1, figsize=plot_options['figsize']) 
    else: _ax = ax
    

    # make sure y and marker are iterable
    if isinstance(y,str): y = [y]
    if marker == None:
        if not isinstance(marker,list): marker = len(y)*[marker]
    else:
        if not isinstance(marker,list): marker =[marker]
    

    for _y,_marker in zip(y,marker):
        label = _y if hue == None else None
        if plot_options['palette'] !=None:
            g = sns.lineplot(data=df,x=x, y=_y, hue=hue, palette=palette,  ax=_ax, label=label,
                             alpha = alpha, errorbar=errorbar, marker=_marker, estimator=estimator,
                             markers=markers, style=style, linestyle=linestyle)
        elif plot_options['color'] !=None:
            g = sns.lineplot(data=df,x=x,y=_y, hue=hue, color=color,  ax=_ax, label=label,
                             alpha = alpha, errorbar=errorbar, marker=_marker, estimator=estimator,
                             markers=markers, style=style, linestyle=linestyle)
        else:
            g = sns.lineplot(data=df,x=x,y=_y, hue=hue, ax=_ax, label=label,
                             alpha= alpha, errorbar=errorbar, marker=_marker, estimator=estimator,
                             markers=None, style=style, linestyle=linestyle)
    
    
    if yfb != None:
        yfb1 = yfb[0]
        yfb2 = yfb[1]
        _ax.fill_between(df[x], df[yfb1], df[yfb2], color = yfb_color, alpha=yfb_alpha)
    
                 #df_ml_pred["unit_sales_pred"]+df_ml_pred["error_lower"] ,
                 #df_ml_pred["unit_sales_pred"]+df_ml_pred["error_upper"] , 
                 #color="red", alpha=0.5)
            
            
    # second y_axis ... plot this first so that primary y is plotted over secondary
    if y2 != None:
        _ax2 = _ax.twinx()
        
        if isinstance(y2,str): 
            y2 = [y2]
        

        
        if marker2 == None:
            if not isinstance(marker2,list): marker2 = len(y2)*[marker2]
        else:
            if not isinstance(marker2,list): marker2 =[marker2]
            
        for _y2,_marker2 in zip(y2,marker2):
            label = _y2 if hue == None else None
            if plot_options['palette2'] !=None:
                g = sns.lineplot(data=df,x=x, y=_y2, hue=hue, palette=palette2,  errorbar=errorbar2, label=label,
                                        ax=_ax2, alpha = alpha2, marker=_marker2,
                                        estimator=estimator2, markers=markers2, style=style2, linestyle=linestyle2)
            elif plot_options['color2'] !=None:
                g = sns.lineplot(data=df,x=x, y=_y2, hue=hue, color=color2,  errorbar=errorbar2, label=label,
                                        ax=_ax2, alpha=alpha2, marker=_marker2,
                                        estimator=estimator2, markers=markers2, style=style2, linestyle=linestyle2)
            else:
                g = sns.lineplot(data=df,x=x, y=_y2, hue=hue, ax=_ax2,  errorbar=errorbar2, label=label,
                                        alpha=alpha2, marker=_marker2,
                                        estimator=estimator2,markers=markers2, style=style2, linestyle=linestyle2) 
                    
            _ax2.grid(visible=None)        
            
    # yaxis format
    # y axis Parameters primary axis
    
    bp.set_axisparams(plot_options,_ax,g)  # axis parameters
    bp.set_yaxis_format(_ax,yaxisformat, ycurrency)
    
    # y2 axis parameters
    if y2 != None:
        plot_options['y_axis_label']=y2label
        bp.set_axisparams(plot_options,_ax2,g)  # axis 2 parameters
        
        # set ylims after set_axis ... set_axis lims defaults to primary y axis 
        if plot_options['y_lims2'] != None:
            _ax2.set_ylim(plot_options['y_lims2'])
        
        # axis 2 legend
        handles, labels = _ax2.get_legend_handles_labels()
        if y2 != None and handles:
            _ax2.legend( loc=plot_options['legend_loc2'], prop={'size': plot_options['legend_fontsize']})

        bp.set_yaxis_format(_ax2,y2axisformat, y2currency)
        
    # plot show if easy lineplot created the figure
    if ax==None and test_mode==False: plt.show() # if simpl_plot created the figure then plt.show()
    
    return None


# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['beautifulplots']

package_data = \
{'': ['*']}

install_requires = \
['matplotlib>=3.4', 'pandas>=1.0', 'seaborn>=0.12']

setup_kwargs = {
    'name': 'beautifulplots',
    'version': '0.2.7',
    'description': 'Python plotting library for beautiful, easy, and sophisticated matplotlib based plots. Compatible with Pandas and Seaborn plots.',
    'long_description': '# beautifulplots\n\nPython plotting library for beautiful, easy, and sophisticated matplotlib based plots. Compatible with Matplotlib graphs, such as Pandas and Seaborn.\n\n## Motivation\nData scientists use data visualization for analysis, investigation, insight, and story telling. In this endeavor, Matplotlib based plotting libraries are reputedly the most powerful and customizable. However, creating graphs for analysis, though necessary, often diverges from the primary goal. Furthermore, Graphs resulting from data science analysis inevitably populate reports, presentations, and even interactive web graphics. After some experience, use case after use case requires similar supporting code. \n\n## Goal\nThe goal of the beautifulplots plotting library is to package graphing functions for the easy generation of beautiful graphs without the burden of distracting code.\n\n## Installation\n\n```bash\n$ pip install beautifulplots\n```\n\n## Usage and Examples\n\n- See usage examples in examples.ipynb (Jupyter notebook) and readthedocs [beatifulplots readthedocs](https://beautifulplots.readthedocs.io/en/latest/index.html).\n\n\n## Feature Requests and Issues\n\nFeature requests and issues: please provide feedback and suggestions for improvements or issues via github issues [new issue](https://github.com/Aljgutier/beautifulplots/issues).\n\n\n## License\n\n`beautifulplots` was created by Alberto Gutierrez. It is licensed under the terms of the MIT license.\n\n## Credits\n\n`beautifulplots` package setup with  [`cookiecutter`](https://cookiecutter.readthedocs.io/en/latest/) and the `py-pkgs-cookiecutter` [template](https://github.com/py-pkgs/py-pkgs-cookiecutter).\n',
    'author': 'Alberto Gutierrez',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9',
}


setup(**setup_kwargs)



import matplotlib.pyplot as plt
import seaborn as sns
from matplotlib.colors import Normalize
import matplotlib as mpl
import  beautifulplots.beautifulplots as bp 

def scatterplot(df, x, y, ax=None, test_mode=False, **kwargs):
    """Scatterplot function. 
    The underlying barplot is ased on the Seaborn with additions, such as data labels,
    and improved default parameters. Refer to beautifulplots plot_defaults for a complete list of options.
    
    Args:
        df (DataFrame): The input DataFrame containing colums corresponding to bar_plot values ("bar_values") and column names (see examples in documentation)
            
        x: Dataframe column corresponding to the  x-axis
        
        y: Column or list of columns corresponding to the y-axis
            
        ax (axis): matplotlib axis (optional), default = None. If axis is None, then create a matplolib figure, axis to host the barplot
            
        color: default = None (Matplotlib default). Matplotlib compatabile color name as text or RGB values, for example, color = [51/235,125/235,183/235].
            
        palette: Matplotlib compatible color palette name, for example, "tab20"
            
        hue: Name of hue dimension variable (i.e., DataFrame column name)
        
        additional options:  see beautifulplot.plot_defaults for additional input variables.
            
    Returns:
        returns True if processing completes succesfully (without errors).
    """

#hline_label=None, vline_label=None,
# annotate
    # get default plot option dictionary
    plot_options = bp.get_kwargs(**kwargs)

    plot_options.update({'y_lims':(.9,100)})
    plot_options.update({'x_lims':(-.05,1.8)})
    
    
    hue = plot_options['hue']
    errorbar = plot_options['errorbar']
    alpha = plot_options['alpha']
    hue = plot_options['hue']
    palette = plot_options['palette']
    color = plot_options['color']
    marker = plot_options['marker']
    markers = plot_options['markers']
    style = plot_options['style']
    ycurrency = plot_options['y_currency']
    ylabel = plot_options['y_axis_label']
    yaxisformat = plot_options['y_axis_format']

    # make sure y and marker are iterable
    if not isinstance(y,list): y = [y]
    if marker == None:
        if not isinstance(marker,list): marker = len(y)*[marker]
    else:
        if not isinstance(marker,list): marker =[marker]

    if ax == None: 
        mpl.rcParams.update(mpl.rcParamsDefault) # reset plot/figure parameters
        plt.style.use(plot_options['pltstyle'])
        fig,_ax = plt.subplots(nrows=1, ncols=1, figsize=plot_options['figsize']) 
    else: _ax = ax

    for _y,_marker in zip(y,marker):
        label = _y if hue == None else None
        if plot_options['palette'] !=None:
            g = sns.scatterplot(data=df,x=x, y=_y, hue=hue, palette=palette,  ax=_ax, label=label,
                             alpha = alpha, marker=_marker,
                             markers=markers, style=style)
        elif plot_options['color'] !=None:
            g = sns.scatterplot(data=df,x=x,y=_y, hue=hue, color=color,  ax=_ax, label=label,
                             alpha = alpha,  marker=_marker, markers=markers, style=style)
        else:

            g = sns.scatterplot(data=df,x=x,y=_y, hue=hue, ax=_ax,label=label,
                             alpha= alpha, marker=_marker,  markers=None, style=style)
            
    
    bp.set_axisparams(plot_options,_ax,g) 
    bp.set_yaxis_format(_ax,yaxisformat, ycurrency)
    
   
    
    # plot show if easy lineplot created the figure
    if ax==None and test_mode==False: plt.show() # if simpl_plot created the figure then plt.show()
        
    return None